@@ -1,16 +1,21 @@
 #ifndef _LIBNETFILTER_ACCT_H_
 #define _LIBNETFILTER_ACCT_H_
 
-#include <sys/types.h>
 #include <stdint.h>
 #include <linux/netfilter/nfnetlink_acct.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 struct nfacct;
 
 enum nfacct_attr_type {
 	NFACCT_ATTR_NAME = 0,
 	NFACCT_ATTR_PKTS,
 	NFACCT_ATTR_BYTES,
+	NFACCT_ATTR_FLAGS,
+	NFACCT_ATTR_QUOTA,
 };
 
 struct nfacct *nfacct_alloc(void);
@@ -36,7 +41,12 @@ int nfacct_nlmsg_parse_payload(const struct nlmsghdr *nlh, struct nfacct *nfacct
 
 #define NFACCT_SNPRINTF_T_PLAIN 0
 #define NFACCT_SNPRINTF_T_XML 1
+#define NFACCT_SNPRINTF_T_JSON 2
 
 int nfacct_snprintf(char *buf, size_t size, struct nfacct *nfacct, uint16_t type, uint16_t flags);
 
+#ifdef __cplusplus
+} /* extern "C" */
+#endif
+
 #endif
